package de.bsvrz.vew.ereigniskal.ereigniskal;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Vector;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dambach.vewdynobj.VerwaltungDynObj;
import de.bsvrz.vew.ereigniskal.ereigniskal.erinnerungsfunktion.Sender;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class SystemTest implements ClientReceiverInterface
{

  /**
   * Datenverteilerverbindung.
   */
  private static ClientDavInterface _connection = null;

  /**
   * Pid Kalender
   */
  private static String _strKalender = "kv.aoe.nw.nba.koeln.koblenz";

  /**
   * Pid Konfigurationsbereich
   */
  private static String _strKonfigBereich = "kb.default.aoe.nw.nba.koeln.koblenz";

  private static Process _prozess = null;

  private static Vector<ResultData> _resultList = null;

  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    ClientDavParameters parameters = new ClientDavParameters();
    parameters.setDavCommunicationAddress("localhost");
    parameters.setDavCommunicationSubAddress(8083);
    parameters.setUserName("Tester");
    parameters.setUserPassword("geheim");

    _connection = new ClientDavConnection(parameters);
    _connection.connect();
    _connection.login();

    // System.out.println("Erzeuge SystemKalenderEintrge");

    erzeugeSystemKalenderEintrag("ske.ostern", "Ostern", "Ostern:=Ostersonntag");
    erzeugeSystemKalenderEintrag("ske.systemdatumA", "SystemdatumA", "SystemdatumA:=Ostern+2Tage");
    erzeugeSystemKalenderEintrag("ske.systemdatumB", "SystemdatumB",
        "SystemdatumB:=<13.04.2004 03:17:00,000-14.04.2004 03:16:59,999>");
    erzeugeSystemKalenderEintrag("ske.systemdatumC", "SystemdatumC",
        "SystemdatumC:=<09.04.2004-16.04.2004>({08:00:00,000-16:59:59,999})");
    erzeugeSystemKalenderEintrag("ske.systemdatumD", "SystemdatumD",
        "SystemdatumD:=<10.04.2004 17:36:00,000-11.04.2004 00:24:59,999>");

    // System.out.println("Erzeuge EreignisTypen");

    erzeugeEreignisTyp("ereignisTyp.dienstagTest", "EreignisTypDienstagTest", 10);
    erzeugeEreignisTyp("ereignisTyp.sportereignis", "EreignisTypSportereignis", 100);
    erzeugeEreignisTyp("ereignisTyp.messe", "EreignisTypMesse", 10);
    erzeugeEreignisTyp("ereignisTyp.unfall", "EreignisTypUnfall", 10);

    // System.out.println("Erzeuge Ereignisse");

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Data data = _connection.createData(atg);
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    ReferenceArray refs = data.getReferenceArray("RumlicheGltigkeit");
    refs.setLength(1);
    SystemObject so = _connection.getDataModel().getObject("ass.p.11903.p.11904.0");
    refs.getReferenceValue(0).setSystemObject(so);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.systemdatumA");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.a", "A", "ereignisTyp.dienstagTest", data);

    data = _connection.createData(atg);
    refs = data.getReferenceArray("RumlicheGltigkeit");
    refs.setLength(1);
    so = _connection.getDataModel().getObject("ass.p.11905.p.11906.0");
    refs.getReferenceValue(0).setSystemObject(so);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.systemdatumB");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    Array array = data.getArray("VerkehrlicheGltigkeit");
    array.setLength(1);
    array.getItem(0).getItem("ZeitDauerAnfangIntervall").asTimeValue().setText("30 Minuten");
    array.getItem(0).getItem("ZeitBezugAnfangIntervall").asTextValue().setText("vorAnfang");
    array.getItem(0).getItem("ZeitDauerEndeIntervall").asTimeValue().setText("15 Minuten");
    array.getItem(0).getItem("ZeitBezugEndeIntervall").asTextValue().setText("vorAnfang");
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.b", "B", "ereignisTyp.sportereignis", data);

    data = _connection.createData(atg);
    refs = data.getReferenceArray("RumlicheGltigkeit");
    refs.setLength(2);
    so = _connection.getDataModel().getObject("ass.p.11903.p.11904.0");
    refs.getReferenceValue(0).setSystemObject(so);
    so = _connection.getDataModel().getObject("ass.p.11904.p.11905.0");
    refs.getReferenceValue(1).setSystemObject(so);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.systemdatumC");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    array = data.getArray("VerkehrlicheGltigkeit");
    array.setLength(1);
    array.getItem(0).getItem("ZeitDauerAnfangIntervall").asTimeValue().setText("30 Minuten");
    array.getItem(0).getItem("ZeitBezugAnfangIntervall").asTextValue().setText("vorAnfang");
    array.getItem(0).getItem("ZeitDauerEndeIntervall").asTimeValue().setText("15 Minuten");
    array.getItem(0).getItem("ZeitBezugEndeIntervall").asTextValue().setText("vorAnfang");
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.c", "C", "ereignisTyp.messe", data);

    data = _connection.createData(atg);
    refs = data.getReferenceArray("RumlicheGltigkeit");
    refs.setLength(2);
    so = _connection.getDataModel().getObject("ass.p.11903.p.11904.0");
    refs.getReferenceValue(0).setSystemObject(so);
    so = _connection.getDataModel().getObject("ass.p.11904.p.11905.0");
    refs.getReferenceValue(1).setSystemObject(so);
    try
    {
      date1 = sdf.parse("13.04.2004 09:37:00,000");
      date2 = sdf.parse("13.04.2004 12:15:59,999");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.systemdatumC");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(null);
    array = data.getArray("VerkehrlicheGltigkeit");
    array.setLength(1);
    array.getItem(0).getItem("ZeitDauerAnfangIntervall").asTimeValue().setText("30 Minuten");
    array.getItem(0).getItem("ZeitBezugAnfangIntervall").asTextValue().setText("vorAnfang");
    array.getItem(0).getItem("ZeitDauerEndeIntervall").asTimeValue().setText("15 Minuten");
    array.getItem(0).getItem("ZeitBezugEndeIntervall").asTextValue().setText("vorAnfang");
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.d", "D", "ereignisTyp.unfall", data);

    data = _connection.createData(atg);
    refs = data.getReferenceArray("RumlicheGltigkeit");
    refs.setLength(1);
    so = _connection.getDataModel().getObject("ass.p.11903.p.11904.0");
    refs.getReferenceValue(0).setSystemObject(so);
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.systemdatumD");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");

    erzeugeEreignis("ereignis.e", "E", "ereignisTyp.messe", data);

    
    new SystemTest().anmeldenEreignisAntwort();

    starteAppliaktionsProzess();

    System.out.println("Warten bis Applikation gestartet...");

    Thread.sleep(20000);

  }

  private static void starteAppliaktionsProzess() throws IOException
  {
    // String separator = System.getProperty("file.separator");
    String separator = "/";
    String javaHome = System.getProperty("java.home");
    String classPath = System.getProperty("java.class.path");

    ClientDavParameters params = _connection.getClientDavParameters();
    String dav = params.getDavCommunicationAddress() + ":" + params.getDavCommunicationSubAddress();
    String user = params.getUserName();

    String cmdLine = javaHome + separator + "bin" + separator + "java";
    cmdLine += " -cp " + "\"" + classPath + "\"" + " -Xmx128m ";
    cmdLine += "de.bsvrz.vew.ereigniskal.ereigniskal.EreignisKalender";
    cmdLine += " -datenverteiler=" + dav;
    cmdLine += " -benutzer=" + user;
    cmdLine += " -authentifizierung=passwd";
    // cmdLine += " -authentifizierung=C:/kernsoftware-3.4.6-test/skripte-dosshell/passwd";
    cmdLine += " -debugLevelStdErrText=CONFIG";
    cmdLine += " -kalender=" + _connection.getLocalConfigurationAuthority().getPid();
    cmdLine += " -neuberechnung=" + "\"" + "01.02.2008 01:00:00,000" + "\"";

    _prozess = Runtime.getRuntime().exec(cmdLine);

    new StdErrCatcher().start();

  }

  private static class StdErrCatcher extends Thread
  {

    byte[] buffer = new byte[1024];

    @Override
    public void run()
    {
      while (true)
      {

        try
        {
          if (_prozess.getErrorStream().available() != 0)
          {
            int anzahl = _prozess.getErrorStream().read(buffer);
            System.out.print("" + new String(buffer, 0, anzahl));

          }
          else
          {
            sleep(100);
          }
        }
        catch (IOException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace(System.out);
        }
        catch (InterruptedException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace(System.out);
        }

      }
    }
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
    if (_prozess != null)
      _prozess.destroy();

    loescheDynamischesObjekt("ereignis.a", "Ereignisse");
    loescheDynamischesObjekt("ereignis.b", "Ereignisse");
    loescheDynamischesObjekt("ereignis.c", "Ereignisse");
    loescheDynamischesObjekt("ereignis.d", "Ereignisse");
    loescheDynamischesObjekt("ereignis.e", "Ereignisse");
    loescheDynamischesObjekt("ereignis.gueltig", "Ereignisse");
    loescheDynamischesObjekt("ereignis.ungueltig", "Ereignisse");

    loescheDynamischesObjekt("ereignisTyp.dienstagTest", "EreignisTypen");
    loescheDynamischesObjekt("ereignisTyp.sportereignis", "EreignisTypen");
    loescheDynamischesObjekt("ereignisTyp.messe", "EreignisTypen");
    loescheDynamischesObjekt("ereignisTyp.unfall", "EreignisTypen");

    loescheDynamischesObjekt("ske.ostern", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.systemdatumA", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.systemdatumB", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.systemdatumC", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.systemdatumD", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.gueltig", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.ungueltig", "SystemKalenderEintrge");

  }

  @Before
  public void setUp() throws Exception
  {
  }

  @After
  public void tearDown() throws Exception
  {
  }

  private void anmeldenEreignisAntwort()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAntwort");
    Aspect asp = _connection.getDataModel().getAspect("asp.antwort");
    DataDescription dd = new DataDescription(atg, asp);

    ClientApplication lao = _connection.getLocalApplicationObject();
    _connection.subscribeReceiver(this, (SystemObject)lao, dd, ReceiveOptions.normal(), ReceiverRole.drain());
  }

  @Test(timeout = 5000)
  public void systemTest1() throws Exception
  {

    // anmeldenEreignisAntwort();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAnfrage");
    Aspect asp = _connection.getDataModel().getAspect("asp.anfrage");
    DataDescription dd = new DataDescription(atg, asp);

    Data data = _connection.createData(atg);
    ClientApplication lao = _connection.getLocalApplicationObject();

    data.getItem("absenderId").asReferenceValue().setSystemObject((SystemObject)lao);
    data.getItem("absenderZeichen").asTextValue().setText("Anfrage 1");
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("11.04.2004 00:00:00,000");
      date2 = sdf.parse("14.04.2004 00:00:00,000");
//      date2 = sdf.parse("13.04.2004 23:59:59,999");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getItem("Anfangszeitpunkt").asTimeValue().setMillis(date1.getTime());
    data.getItem("Endzeitpunkt").asTimeValue().setMillis(date2.getTime());
    data.getItem("EreignisTypenOption").asTextValue().setText("Alle");

    ResultData rd = new ResultData(_connection.getLocalConfigurationAuthority(), dd, new Date().getTime(), data);
    Sender sender = new Sender(_connection, SenderRole.sender());

    _resultList = new Vector<ResultData>();

    sender.sende(rd, dd);

    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      if (_resultList.size() < 1)
      {
        _resultList.wait();
      }
    }

    ResultData resultData = _resultList.get(0);

    data = resultData.getData();

    String s = data.getItem("absenderZeichen").asTextValue().getText();

    assertEquals("Test 1", "Anfrage 1", s);

    s = data.getItem("nderung").asTextValue().getText();

    assertEquals("Test 2", "Ja", s);

    Array array = data.getItem("Ereignis").asArray();

    String[] strings = new String[array.getLength()];

    for (int i = 0; i < array.getLength(); i++)
    {
      Data data2 = array.getItem(i);

      long l = data2.getItem("Zeitpunkt").asTimeValue().getMillis();
      Date d = new Date();
      d.setTime(l);
      String s1 = sdf.format(d);

      String s2 = data2.getItem("EreignisReferenz").asReferenceValue().getSystemObject().getPid();
      String s3 = data2.getItem("zeitlichGltig").asTextValue().getText();
      String s4 = data2.getItem("verkehrlichGltig").asTextValue().getText();

      strings[i] = s1 + " " + s2 + " " + s3 + " " + s4;

      System.out.println(strings[i]);

    }

    System.out.println(array.getLength());

    assertTrue("Test 3", array.getLength() == 22);    
    assertEquals("Test 4", "11.04.2004 00:25:00,000 ereignis.e Nein Nein", strings[0]);
    assertEquals("Test 5", "11.04.2004 07:30:00,000 ereignis.c Nein Ja", strings[1]);
    assertEquals("Test 6", "11.04.2004 07:45:00,000 ereignis.c Nein Nein", strings[2]);
    assertEquals("Test 7", "11.04.2004 08:00:00,000 ereignis.c Ja Nein", strings[3]);
    assertEquals("Test 9", "11.04.2004 17:00:00,000 ereignis.c Nein Nein", strings[4]);
    assertEquals("Test 9", "12.04.2004 07:30:00,000 ereignis.c Nein Ja", strings[5]);
    assertEquals("Test 10", "12.04.2004 07:45:00,000 ereignis.c Nein Nein", strings[6]);
    assertEquals("Test 11", "12.04.2004 08:00:00,000 ereignis.c Ja Nein", strings[7]);
    assertEquals("Test 12", "12.04.2004 17:00:00,000 ereignis.c Nein Nein", strings[8]);
    assertEquals("Test 13", "13.04.2004 00:00:00,000 ereignis.a Ja Nein", strings[9]);
    assertEquals("Test 14", "13.04.2004 02:47:00,000 ereignis.b Nein Ja", strings[10]);
    assertEquals("Test 15", "13.04.2004 03:02:00,000 ereignis.b Nein Nein", strings[11]);
    assertEquals("Test 16", "13.04.2004 03:17:00,000 ereignis.b Ja Nein", strings[12]);
    assertEquals("Test 17", "13.04.2004 07:30:00,000 ereignis.c Nein Ja", strings[13]);
    assertEquals("Test 18", "13.04.2004 07:45:00,000 ereignis.c Nein Nein", strings[14]);
    assertEquals("Test 19", "13.04.2004 08:00:00,000 ereignis.c Ja Nein", strings[15]);
    assertEquals("Test 20", "13.04.2004 09:07:00,000 ereignis.d Nein Ja", strings[16]);
    assertEquals("Test 21", "13.04.2004 09:22:00,000 ereignis.d Nein Nein", strings[17]);
    assertEquals("Test 22", "13.04.2004 09:37:00,000 ereignis.d Ja Nein", strings[18]);
    assertEquals("Test 23", "13.04.2004 12:16:00,000 ereignis.d Nein Nein", strings[19]);
    assertEquals("Test 24", "13.04.2004 17:00:00,000 ereignis.c Nein Nein", strings[20]);
    assertEquals("Test 25", "14.04.2004 00:00:00,000 ereignis.a Nein Nein", strings[21]);

  }

  @Test(timeout = 5000)
  public void systemTest2() throws Exception
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAnfrage");
    Aspect asp = _connection.getDataModel().getAspect("asp.anfrage");
    DataDescription dd = new DataDescription(atg, asp);

    Data data = _connection.createData(atg);
    ClientApplication lao = _connection.getLocalApplicationObject();

    data.getItem("absenderId").asReferenceValue().setSystemObject((SystemObject)lao);
    data.getItem("absenderZeichen").asTextValue().setText("Anfrage 2");
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("11.04.2004 00:00:00,000");
      date2 = sdf.parse("14.04.2004 00:00:00,000");
//      date2 = sdf.parse("13.04.2004 23:59:59,999");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getItem("Anfangszeitpunkt").asTimeValue().setMillis(date1.getTime());
    data.getItem("Endzeitpunkt").asTimeValue().setMillis(date2.getTime());
    data.getItem("EreignisTypenOption").asTextValue().setText("Nicht");
    Array array = data.getArray("EreignisTypReferenz");
    array.setLength(1);
    SystemObject so = _connection.getDataModel().getObject("ereignisTyp.messe");
    array.getItem(0).asReferenceValue().setSystemObject(so);

    ResultData rd = new ResultData(_connection.getLocalConfigurationAuthority(), dd, new Date().getTime(), data);
    Sender sender = new Sender(_connection, SenderRole.sender());

    _resultList.clear();

    sender.sende(rd, dd);
    
    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      if (_resultList.size() < 1)
      {
        _resultList.wait();
      }
    }
    
    ResultData resultData = _resultList.get(0);

    data = resultData.getData();

    String s = data.getItem("absenderZeichen").asTextValue().getText();

    assertEquals("Test 27", "Anfrage 2", s);

    s = data.getItem("nderung").asTextValue().getText();

    assertEquals("Test 28", "Ja", s);

    array = data.getItem("Ereignis").asArray();

    String[] strings = new String[array.getLength()];

    for (int i = 0; i < array.getLength(); i++)
    {
      Data data2 = array.getItem(i);

      long l = data2.getItem("Zeitpunkt").asTimeValue().getMillis();
      Date d = new Date();
      d.setTime(l);
      String s1 = sdf.format(d);

      String s2 = data2.getItem("EreignisReferenz").asReferenceValue().getSystemObject().getPid();
      String s3 = data2.getItem("zeitlichGltig").asTextValue().getText();
      String s4 = data2.getItem("verkehrlichGltig").asTextValue().getText();

      strings[i] = s1 + " " + s2 + " " + s3 + " " + s4;

//       System.out.println(strings[i]);

    }

    System.out.println(array.getLength());

    assertTrue("Test 29", array.getLength() == 9);
    assertEquals("Test 30", "13.04.2004 00:00:00,000 ereignis.a Ja Nein", strings[0]);
    assertEquals("Test 31", "13.04.2004 02:47:00,000 ereignis.b Nein Ja", strings[1]);
    assertEquals("Test 32", "13.04.2004 03:02:00,000 ereignis.b Nein Nein", strings[2]);
    assertEquals("Test 33", "13.04.2004 03:17:00,000 ereignis.b Ja Nein", strings[3]);
    assertEquals("Test 34", "13.04.2004 09:07:00,000 ereignis.d Nein Ja", strings[4]);
    assertEquals("Test 35", "13.04.2004 09:22:00,000 ereignis.d Nein Nein", strings[5]);
    assertEquals("Test 36", "13.04.2004 09:37:00,000 ereignis.d Ja Nein", strings[6]);
    assertEquals("Test 37", "13.04.2004 12:16:00,000 ereignis.d Nein Nein", strings[7]);
    assertEquals("Test 38", "14.04.2004 00:00:00,000 ereignis.a Nein Nein", strings[8]);

  }

  @Test(timeout = 5000)
  public void systemTest3() throws Exception
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAnfrage");
    Aspect asp = _connection.getDataModel().getAspect("asp.anfrage");
    DataDescription dd = new DataDescription(atg, asp);

    Data data = _connection.createData(atg);
    ClientApplication lao = _connection.getLocalApplicationObject();

    data.getItem("absenderId").asReferenceValue().setSystemObject((SystemObject)lao);
    data.getItem("absenderZeichen").asTextValue().setText("Anfrage 3");

    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("11.04.2004 00:00:00,000");
      date2 = sdf.parse("13.04.2004 23:59:59,999");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getItem("Anfangszeitpunkt").asTimeValue().setMillis(date1.getTime());
    data.getItem("Endzeitpunkt").asTimeValue().setMillis(date2.getTime());

    Array array = data.getArray("RumlicheGltigkeit");
    array.setLength(2);
    SystemObject so = _connection.getDataModel().getObject("ass.p.11903.p.11904.0");
    array.getItem(0).asReferenceValue().setSystemObject(so);
    so = _connection.getDataModel().getObject("ass.p.11904.p.11905.0");
    array.getItem(1).asReferenceValue().setSystemObject(so);

    data.getItem("EreignisTypenOption").asTextValue().setText("Nicht");

    array = data.getArray("EreignisTypReferenz");
    array.setLength(1);
    so = _connection.getDataModel().getObject("ereignisTyp.messe");
    array.getItem(0).asReferenceValue().setSystemObject(so);

    ResultData rd = new ResultData(_connection.getLocalConfigurationAuthority(), dd, new Date().getTime(), data);
    Sender sender = new Sender(_connection, SenderRole.sender());

    _resultList = new Vector<ResultData>();

    sender.sende(rd, dd);

    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      if (_resultList.size() < 1)
      {
        _resultList.wait();
      }
    }

    ResultData resultData = _resultList.get(0);

    data = resultData.getData();

    String s = data.getItem("absenderZeichen").asTextValue().getText();

    assertEquals("Test 39", "Anfrage 3", s);

    s = data.getItem("nderung").asTextValue().getText();

    assertEquals("Test 40", "Ja", s);

    array = data.getItem("Ereignis").asArray();

    String[] strings = new String[array.getLength()];

    for (int i = 0; i < array.getLength(); i++)
    {
      Data data2 = array.getItem(i);

      long l = data2.getItem("Zeitpunkt").asTimeValue().getMillis();
      Date d = new Date();
      d.setTime(l);
      String s1 = sdf.format(d);

      String s2 = data2.getItem("EreignisReferenz").asReferenceValue().getSystemObject().getPid();
      String s3 = data2.getItem("zeitlichGltig").asTextValue().getText();
      String s4 = data2.getItem("verkehrlichGltig").asTextValue().getText();

      strings[i] = s1 + " " + s2 + " " + s3 + " " + s4;

      // System.out.println(strings[i]);

    }

    System.out.println(array.getLength());

    assertTrue("Test 41", array.getLength() == 6);
    assertEquals("Test 42", "13.04.2004 00:00:00,000 ereignis.a Ja Nein", strings[0]);
    assertEquals("Test 43", "13.04.2004 09:07:00,000 ereignis.d Nein Ja", strings[1]);
    assertEquals("Test 44", "13.04.2004 09:22:00,000 ereignis.d Nein Nein", strings[2]);
    assertEquals("Test 45", "13.04.2004 09:37:00,000 ereignis.d Ja Nein", strings[3]);
    assertEquals("Test 46", "13.04.2004 12:16:00,000 ereignis.d Nein Nein", strings[4]);
    assertEquals("Test 47", "14.04.2004 00:00:00,000 ereignis.a Nein Nein", strings[5]);

  }

  @Test(timeout = 6000)
  public void systemTest4() throws Exception
  {
    // Ereignis umparametrieren
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Data data = _connection.createData(atg);
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    ReferenceArray refs = data.getReferenceArray("RumlicheGltigkeit");
    refs.setLength(1);
    SystemObject so = _connection.getDataModel().getObject("ass.p.11905.p.11906.0");
    refs.getReferenceValue(0).setSystemObject(so);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.systemdatumA");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");

    // Erzeuge Parameter
    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignis");

    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);

    Data[] datas = new Data[1];
    datas[0] = data;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject("ereignis.a"));
    vewParam.parametriere(datas);

    // Warten bis die Parametrierung die nderung publiziert
    Thread.sleep(1000);

    // Anfrage erstellen
    atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAnfrage");
    Aspect asp = _connection.getDataModel().getAspect("asp.anfrage");
    DataDescription dd = new DataDescription(atg, asp);

    data = _connection.createData(atg);
    ClientApplication lao = _connection.getLocalApplicationObject();

    data.getItem("absenderId").asReferenceValue().setSystemObject((SystemObject)lao);
    data.getItem("absenderZeichen").asTextValue().setText("Anfrage 4");

    sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    date1 = null;
    date2 = null;
    try
    {
      date1 = sdf.parse("11.04.2004 00:00:00,000");
      date2 = sdf.parse("13.04.2004 23:59:59,999");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getItem("Anfangszeitpunkt").asTimeValue().setMillis(date1.getTime());
    data.getItem("Endzeitpunkt").asTimeValue().setMillis(date2.getTime());

    Array array = data.getArray("RumlicheGltigkeit");
    array.setLength(2);
    so = _connection.getDataModel().getObject("ass.p.11903.p.11904.0");
    array.getItem(0).asReferenceValue().setSystemObject(so);
    so = _connection.getDataModel().getObject("ass.p.11904.p.11905.0");
    array.getItem(1).asReferenceValue().setSystemObject(so);

    data.getItem("EreignisTypenOption").asTextValue().setText("Nicht");

    array = data.getArray("EreignisTypReferenz");
    array.setLength(1);
    so = _connection.getDataModel().getObject("ereignisTyp.messe");
    array.getItem(0).asReferenceValue().setSystemObject(so);

    ResultData rd = new ResultData(_connection.getLocalConfigurationAuthority(), dd, new Date().getTime(), data);
    Sender sender = new Sender(_connection, SenderRole.sender());

    _resultList.clear();

    sender.sende(rd, dd);

    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      if (_resultList.size() < 1)
      {
        _resultList.wait();
      }
    }

    ResultData resultData = _resultList.get(0);

    data = resultData.getData();

    String s = data.getItem("absenderZeichen").asTextValue().getText();

    assertEquals("Test 48", "Anfrage 4", s);

    s = data.getItem("nderung").asTextValue().getText();

    assertEquals("Test 49", "Ja", s);

    array = data.getItem("Ereignis").asArray();

    String[] strings = new String[array.getLength()];

    for (int i = 0; i < array.getLength(); i++)
    {
      Data data2 = array.getItem(i);

      long l = data2.getItem("Zeitpunkt").asTimeValue().getMillis();
      Date d = new Date();
      d.setTime(l);
      String s1 = sdf.format(d);

      String s2 = data2.getItem("EreignisReferenz").asReferenceValue().getSystemObject().getPid();
      String s3 = data2.getItem("zeitlichGltig").asTextValue().getText();
      String s4 = data2.getItem("verkehrlichGltig").asTextValue().getText();

      strings[i] = s1 + " " + s2 + " " + s3 + " " + s4;

      // System.out.println(strings[i]);

    }

    System.out.println(array.getLength());

    assertTrue("Test 50", array.getLength() == 4);
    assertEquals("Test 51", "13.04.2004 09:07:00,000 ereignis.d Nein Ja", strings[0]);
    assertEquals("Test 52", "13.04.2004 09:22:00,000 ereignis.d Nein Nein", strings[1]);
    assertEquals("Test 53", "13.04.2004 09:37:00,000 ereignis.d Ja Nein", strings[2]);
    assertEquals("Test 54", "13.04.2004 12:16:00,000 ereignis.d Nein Nein", strings[3]);

  }

  private void anmeldenEreignisZustand()
  {

    List<SystemObject> list = new ArrayList<SystemObject>();

    SystemObject o1 = _connection.getDataModel().getObject("ereignis.gueltig");
    SystemObject o2 = _connection.getDataModel().getObject("ereignis.ungueltig");

    list.add(o1);
    list.add(o2);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignis");
    Aspect asp = _connection.getDataModel().getAspect("asp.zustand");
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, list, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
  }

  @Test(timeout = 10000)
  public void setUpSystemTest5() throws Exception
  {
    
    System.out.println("SystemTest.setUpSystemTest5(!!!)");
    
    Calendar cal = new GregorianCalendar().getInstance();

    int dow = cal.get(Calendar.DAY_OF_WEEK);

    String defGueltig = null;
    String defUngueltig = null;

    switch (dow)
    {
      case 0:
        defGueltig = "Samstag";
        defUngueltig = "Sonntag";
        break;
      case 1:
        defGueltig = "Sonntag";
        defUngueltig = "Montag";
        break;
      case 2:
        defGueltig = "Montag";
        defUngueltig = "Dienstag";
        break;
      case 3:
        defGueltig = "Dienstag";
        defUngueltig = "Mittwoch";
        break;
      case 4:
        defGueltig = "Mittwoch";
        defUngueltig = "Donnerstag";
        break;
      case 5:
        defGueltig = "Donnerstag";
        defUngueltig = "Freitag";
        break;
      case 6:
        defGueltig = "Freitag";
        defUngueltig = "Samstag";
        break;

      default:
        fail("setUpBeforeClass()");

    }

    erzeugeSystemKalenderEintrag("ske.gueltig", "Gueltig", defGueltig);
    erzeugeSystemKalenderEintrag("ske.ungueltig", "Ungueltig", defUngueltig);

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Data data = _connection.createData(atg);
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    SystemObject so = _connection.getDataModel().getObject("ske.gueltig");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");

    erzeugeEreignis("ereignis.gueltig", "Gueltig", "ereignisTyp.messe", data);

    data = _connection.createData(atg);
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.ungueltig");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");

    erzeugeEreignis("ereignis.ungueltig", "Ungueltig", "ereignisTyp.unfall", data);

    Thread.sleep(5000);
  }

  @Test(timeout = 5000)
  public void systemTest5() throws Exception
  {

    System.out.println("SystemTest.systemTest5(!!!)");
    
    anmeldenEreignisZustand();

    _resultList.clear();

    String gueltig = null;
    String ungueltig = null;

    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 2)
      {
        _resultList.wait();
      }

      for (ResultData rd : _resultList)
      {
        Data d = rd.getData();
        if (rd.getObject().getPid().equals("ereignis.gueltig"))
          gueltig = rd.getObject().getPid() + " " + d.getItem("zeitlichGltig").asTextValue().getText();
        else if (rd.getObject().getPid().equals("ereignis.ungueltig"))
          ungueltig = rd.getObject().getPid() + " " + d.getItem("zeitlichGltig").asTextValue().getText();
      }

    }

    assertEquals("Test 55", "ereignis.gueltig Ja", gueltig);
    assertEquals("Test 56", "ereignis.ungueltig Nein", ungueltig);

  }

  @Test(timeout = 5000)
  public void systemTest6() throws Exception
  {
    // Ereignis umparametrieren
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Data data = _connection.createData(atg);
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    SystemObject so = _connection.getDataModel().getObject("ske.ungueltig");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");

    // Erzeuge Parameter
    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignis");

    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);

    Data[] datas = new Data[1];
    datas[0] = data;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject("ereignis.gueltig"));
    vewParam.parametriere(datas);

    atg = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    data = _connection.createData(atg);
    sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.gueltig");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");

    // Erzeuge Parameter
    datas[0] = data;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject("ereignis.ungueltig"));
    vewParam.parametriere(datas);

    _resultList.clear();

    String gueltig = null;
    String ungueltig = null;

    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 2)
      {
        _resultList.wait();
      }

      for (ResultData rd : _resultList)
      {
        Data d = rd.getData();
        if (rd.getObject().getPid().equals("ereignis.gueltig"))
          gueltig = rd.getObject().getPid() + " " + d.getItem("zeitlichGltig").asTextValue().getText();
        else if (rd.getObject().getPid().equals("ereignis.ungueltig"))
          ungueltig = rd.getObject().getPid() + " " + d.getItem("zeitlichGltig").asTextValue().getText();
      }

    }

    assertEquals("Test 57", "ereignis.gueltig Nein", gueltig);
    assertEquals("Test 58", "ereignis.ungueltig Ja", ungueltig);

  }

  @Test(timeout = 5000)
  public void systemTest7() throws Exception
  {

    // Erzeuge Parameter
    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignis");

    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.ereignisEigenschaften");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);

    vewKonfig.setDynamicObject((DynamicObject)_connection.getDataModel().getObject("ereignis.ungueltig"));
    vewKonfig.entferneVonMenge("Ereignisse");

    // loescheDynamischesObjekt("ereignis.ungueltig", "Ereignisse");

    _resultList.clear();

    String ungueltig = null;

    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      while (_resultList.size() < 1)
      {
        _resultList.wait();
      }

      ResultData rd = _resultList.get(0);
      Data d = rd.getData();
      ungueltig = rd.getObject().getPid() + " " + d.getItem("zeitlichGltig").asTextValue().getText();

    }

    assertEquals("Test 59", "ereignis.ungueltig Nein", ungueltig);

  }

  private static void erzeugeSystemKalenderEintrag(String pid, String name, String definition)
  {

    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.systemKalenderEintrag");

    // Erzeuge dynamisches Objekt
    AttributeGroup atgKonfig = _connection.getDataModel().getAttributeGroup("atg.systemKalenderEintrag");
    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    Data data = _connection.createData(atgKonfig);
    Data[] datas = new Data[1];
    datas[0] = data;
    vewKonfig.erzeuge(pid, name, "SystemKalenderEintrge", null);

    // Erzeuge Parameter
    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.systemKalenderEintrag");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);
    data = _connection.createData(atgParam);
    data.getTextValue("Definition").setText(definition);
    datas[0] = data;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject(pid));
    vewParam.parametriere(datas);
  }

  private static void erzeugeEreignisTyp(String pid, String name, int prio)
  {

    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignisTyp");

    // Erzeuge dynamisches Objekt
    AttributeGroup atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisTypEigenschaften");
    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    Data data = _connection.createData(atgKonfig);
    Array array = data.getItem("ZustzlicheAttribute").asArray();
    array.setLength(1);
    Data d = array.getItem(0);
    d.getTextValue("Attributname").setText("");
    d.getTextValue("Attributwert").setText("");
    Data[] datas = new Data[1];
    datas[0] = data;
    vewKonfig.erzeuge(pid, name, "EreignisTypen", datas);

    // Erzeuge Parameter
    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.ereignisTypParameter");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);
    data = _connection.createData(atgParam);
    data.getScaledValue("EreignisTypPrioritt").set(prio);
    datas[0] = data;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject(pid));
    vewParam.parametriere(datas);

  }

  private static void erzeugeEreignis(String pid, String name, String ereignisTyp, Data parameter)
  {

    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignis");

    // Erzeuge dynamisches Objekt
    AttributeGroup atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisEigenschaften");
    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    Data data = _connection.createData(atgKonfig);
    data.getItem("Ereignisbeschreibung").asTextValue().setText("");
    SystemObject so = _connection.getDataModel().getObject(ereignisTyp);
    data.getItem("EreignisTypReferenz").asReferenceValue().setSystemObject(so);
    Data[] datas = new Data[1];
    datas[0] = data;
    vewKonfig.erzeuge(pid, name, "Ereignisse", datas);

    // Erzeuge Parameter
    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);

    datas[0] = parameter;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject(pid));
    vewParam.parametriere(datas);
  }

  private static void loescheDynamischesObjekt(String pid, String menge)
  {
    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = null;
    AttributeGroup atgKonfig = null;
    if (menge.equals("SystemKalenderEintrge"))
    {
      dot = (DynamicObjectType)_connection.getDataModel().getType("typ.systemKalenderEintrag");
      atgKonfig = _connection.getDataModel().getAttributeGroup("atg.systemKalenderEintrag");
    }
    else if (menge.equals("EreignisTypen"))
    {
      dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignisTyp");
      atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisTypEigenschaften");
    }
    else if (menge.equals("Ereignisse"))
    {
      dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignis");
      atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisEigenschaften");
    }
    else
      fail("Falscher Typ");

    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    vewKonfig.setDynamicObject(_connection.getDataModel().getObject(pid));
    vewKonfig.loesche(pid, menge);
  }

  public void update(ResultData[] results)
  {
    // TODO Auto-generated method stub
    for (ResultData result : results)
    {
      if (result.getData() != null)
      {

        DataDescription dd = result.getDataDescription();
        AttributeGroup atg = dd.getAttributeGroup();

        if (atg.getPid().equals("atg.ereignis") || atg.getPid().equals("atg.ereignisKalenderAntwort"))
        {

//          System.out.println("OK: " + result.getObject() + " " + result.getDataDescription());

          synchronized (_resultList)
          {
            _resultList.add(result);
            _resultList.notify();
          }
        }
        else
        {
//          System.out.println("Atg: " + result.getObject() + " " + result.getDataDescription());
        }

      }
      else
      {
//        System.out.println("Null: " + result.getObject() + " " + result.getDataDescription());
      }
    }
  }

}
