package de.bsvrz.vew.ereigniskal.ereigniskal.ereignis;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Formatter;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.SortedMap;
import java.util.Map.Entry;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.vew.ereigniskal.ereigniskal.EreignisKalender;
import de.bsvrz.vew.ereigniskal.ereigniskal.Parser;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

public class ListeZustandsWechselTest
{
  private static Parser _parser = new Parser();

  private static ListeZustandsWechsel _listeZustandsWechsel = new ListeZustandsWechsel();

  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    // _parser.parseSystemkalenderEintrag("ske.montag", "Montag", "Montag");
    // _parser.parseSystemkalenderEintrag("ske.dienstag", "Dienstag", "Dienstag");
    // _parser.parseSystemkalenderEintrag("ske.mittwoch", "Mittwoch", "Mittwoch");
    // _parser.parseSystemkalenderEintrag("ske.donnerstag", "Donnerstag", "Donnerstag");
    // _parser.parseSystemkalenderEintrag("ske.freitag", "Freitag", "Freitag");
    // _parser.parseSystemkalenderEintrag("ske.samstag", "Samstag", "Samstag");
    // _parser.parseSystemkalenderEintrag("ske.sonntag", "Sonntag", "Sonntag");
    // _parser.parseSystemkalenderEintrag("ske.tag", "Tag", "Tag");
    // _parser.parseSystemkalenderEintrag("ske.ostersonntag", "Ostersonntag", "Ostersonntag");
    // _parser.parseSystemkalenderEintrag("ske.bereich1", "Bereich1",
    // "Bereich1:=<01.01.2008 00:00:00,000-31.01.2008 23:59:59,999>");
    // _parser.parseSystemkalenderEintrag("ske.bereich2", "Bereich2",
    // "Bereich2:=<15.01.2008 00:00:00,000-15.02.2008 23:59:59,999>");

    _parser.parseSystemkalenderEintrag("ske.hauptverkehrszeit", "Hauptverkehrszeit",
        "Hauptverkehrszeit:=({07:15:00,000-08:59:59,999}{15:30:00,000-17:44:59,999})*,*");
    
    _parser.parseSystemkalenderEintrag("ske.hauptverkehrszeit2", "Hauptverkehrszeit2",
        "Hauptverkehrszeit2:=<15.01.2009-16.01.2009>({07:15:00,000-08:59:59,999}{15:30:00,000-17:44:59,999})");
    
    //"Bereich4:=<15.01.2008-15.02.2008>({09:00:00,000-11:59:59,999}{15:30:00,000-17:59:59,999})"
    
    // _parser.parseSystemkalenderEintrag("ske.systemdatumB", "SystemdatumB",
    // "SystemdatumB:=<13.04.2004 03:17:00,000-14.04.2004 03:16:59,999>");
    _parser.parseSystemkalenderEintrag("ske.b", "B", "B:=<13.04.2004 03:17:00,000-14.04.2004 03:16:59,999>");

  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {

  }

  @Before
  public void setUp() throws Exception
  {
    _listeZustandsWechsel = new ListeZustandsWechsel();
  }

  @After
  public void tearDown() throws Exception
  {

  }

  @Test
  public void berechneNaechstenZustandsWechselTest() throws Exception
  {
    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit");

    _listeZustandsWechsel.setListeZustandsWechsel(ske.getListeZustandsWechsel());

    Calendar cal = new GregorianCalendar();

    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    int day = cal.get(Calendar.DAY_OF_MONTH);
    int mon = cal.get(Calendar.MONTH) + 1;
    int year = cal.get(Calendar.YEAR);

    Entry<Long, Boolean> entry = _listeZustandsWechsel.berechneNaechstenZustandsWechsel(cal.getTimeInMillis());

    // SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss,SSS");
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 1", obj + " 07:15:00,000 true", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 8);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneNaechstenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      // System.out.println(s);

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 2", obj + " 08:59:59,999 false", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 12);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneNaechstenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 3", obj + " 15:30:00,000 true", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 16);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneNaechstenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      // System.out.println(s);

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 4", obj + " 17:44:59,999 false", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 18);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneNaechstenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      // System.out.println(s);

      Formatter fmt = new Formatter();

      cal.add(Calendar.DAY_OF_MONTH, 1);

      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", cal.get(Calendar.DAY_OF_MONTH), cal.get(Calendar.MONTH) + 1, cal
          .get(Calendar.YEAR));

      assertEquals("Test 5", obj + " 07:15:00,000 true", s);

    }
  }

  @Test
  public void berechneLetztenZustandsWechselTest() throws Exception
  {
    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit");

    _listeZustandsWechsel.setListeZustandsWechsel(ske.getListeZustandsWechsel());

    Calendar cal = new GregorianCalendar().getInstance();

    cal.set(Calendar.HOUR_OF_DAY, 0);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    int day = cal.get(Calendar.DAY_OF_MONTH);
    int mon = cal.get(Calendar.MONTH) + 1;
    int year = cal.get(Calendar.YEAR);

    Entry<Long, Boolean> entry = _listeZustandsWechsel.berechneLetztenZustandsWechsel(cal.getTimeInMillis());

    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day - 1, mon, year);

      assertEquals("Test 6", obj + " 17:44:59,999 false", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 8);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneLetztenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 7", obj + " 07:15:00,000 true", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 12);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneLetztenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 8", obj + " 08:59:59,999 false", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 16);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneLetztenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 9", obj + " 15:30:00,000 true", s);

    }

    cal.set(Calendar.HOUR_OF_DAY, 18);
    cal.set(Calendar.MINUTE, 0);
    cal.set(Calendar.SECOND, 0);
    cal.set(Calendar.MILLISECOND, 0);

    entry = _listeZustandsWechsel.berechneLetztenZustandsWechsel(cal.getTimeInMillis());

    if (entry != null)
    {
      Date d = new Date();
      d.setTime(entry.getKey());
      String s = sdf.format(d) + " " + entry.getValue();

      Formatter fmt = new Formatter();
      Object obj = fmt.format("%1$02d.%2$02d.%3$02d", day, mon, year);

      assertEquals("Test 10", obj + " 17:44:59,999 false", s);

    }

  }

  @Test
  public void berechneVonBisTest() throws Exception
  {
    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit");
    
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    Date d1 = sdf.parse("15.01.2009 00:00:00,000");
    Date d2 = sdf.parse("16.01.2009 23:59:59,999");
    
    _listeZustandsWechsel.setListeZustandsWechsel(ske.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime()));
    
    SortedMap<Long, Boolean> sm = _listeZustandsWechsel.berechneVonBis(d1.getTime(), d2.getTime());

    assertTrue("Test 10", sm != null);

    String[] strings = new String[sm.entrySet().size()];

    int cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();

      cnt++;
    }

    assertTrue("Test 11", strings.length == 8);
    assertEquals("Test 12", "15.01.2009 07:15:00,000 true", strings[0]);
    assertEquals("Test 13", "15.01.2009 08:59:59,999 false", strings[1]);
    assertEquals("Test 14", "15.01.2009 15:30:00,000 true", strings[2]);
    assertEquals("Test 15", "15.01.2009 17:44:59,999 false", strings[3]);
    assertEquals("Test 16", "16.01.2009 07:15:00,000 true", strings[4]);
    assertEquals("Test 17", "16.01.2009 08:59:59,999 false", strings[5]);
    assertEquals("Test 18", "16.01.2009 15:30:00,000 true", strings[6]);
    assertEquals("Test 19", "16.01.2009 17:44:59,999 false", strings[7]);

  }

  @Test
  public void setGueltigkeitVerkehrTest1() throws Exception
  {
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit2");
        
    Date d1 = sdf.parse("15.01.2009 00:00:00,000");
    Date d2 = sdf.parse("16.01.2009 23:59:59,999");
    
    SortedMap<Long, Boolean> tmp = ske.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime());
    
    SortedMap<Long, Boolean> sm = _listeZustandsWechsel.setGueltigkeitVerkehr(tmp, 60 * 1000,
        0, 60 * 1000, 2);

    String[] strings = new String[sm.entrySet().size()];

    int cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();
      
      cnt++;
    }
    
    assertTrue("Test 20", strings.length == 8);
    assertEquals("Test 21", "15.01.2009 07:14:00,000 true", strings[0]);
    assertEquals("Test 22", "15.01.2009 08:58:59,999 false", strings[1]);
    assertEquals("Test 23", "15.01.2009 15:29:00,000 true", strings[2]);
    assertEquals("Test 24", "15.01.2009 17:43:59,999 false", strings[3]);
    assertEquals("Test 25", "16.01.2009 07:14:00,000 true", strings[4]);
    assertEquals("Test 26", "16.01.2009 08:58:59,999 false", strings[5]);
    assertEquals("Test 27", "16.01.2009 15:29:00,000 true", strings[6]);
    assertEquals("Test 28", "16.01.2009 17:43:59,999 false", strings[7]);

  }

  @Test
  public void setGueltigkeitVerkehrTest2() throws Exception
  {
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit2");

    Date d1 = sdf.parse("15.01.2009 00:00:00,000");
    Date d2 = sdf.parse("16.01.2009 23:59:59,999");
    
    SortedMap<Long, Boolean> tmp = ske.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime());
        
    SortedMap<Long, Boolean> sm = _listeZustandsWechsel.setGueltigkeitVerkehr(tmp, 60 * 1000,
        0, 60 * 1000, 3);

    String[] strings = new String[sm.entrySet().size()];

    int cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();

      cnt++;
    }

    assertTrue("Test 29", strings.length == 8);
    assertEquals("Test 30", "15.01.2009 07:14:00,000 true", strings[0]);
    assertEquals("Test 31", "15.01.2009 09:00:59,999 false", strings[1]);
    assertEquals("Test 32", "15.01.2009 15:29:00,000 true", strings[2]);
    assertEquals("Test 33", "15.01.2009 17:45:59,999 false", strings[3]);
    assertEquals("Test 34", "16.01.2009 07:14:00,000 true", strings[4]);
    assertEquals("Test 35", "16.01.2009 09:00:59,999 false", strings[5]);
    assertEquals("Test 36", "16.01.2009 15:29:00,000 true", strings[6]);
    assertEquals("Test 37", "16.01.2009 17:45:59,999 false", strings[7]);

  }

  @Test
  public void setGueltigkeitVerkehrTest3() throws Exception
  {
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit2");

    Date d1 = sdf.parse("15.01.2009 00:00:00,000");
    Date d2 = sdf.parse("16.01.2009 23:59:59,999");
    
    SortedMap<Long, Boolean> tmp = ske.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime());
    
    SortedMap<Long, Boolean> sm = _listeZustandsWechsel.setGueltigkeitVerkehr(tmp, 60 * 1000,
        1, 60 * 1000, 2);

    String[] strings = new String[sm.entrySet().size()];

    int cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();

      cnt++;
    }

    assertTrue("Test 38", strings.length == 8);
    assertEquals("Test 39", "15.01.2009 07:16:00,000 true", strings[0]);
    assertEquals("Test 40", "15.01.2009 08:58:59,999 false", strings[1]);
    assertEquals("Test 41", "15.01.2009 15:31:00,000 true", strings[2]);
    assertEquals("Test 42", "15.01.2009 17:43:59,999 false", strings[3]);
    assertEquals("Test 43", "16.01.2009 07:16:00,000 true", strings[4]);
    assertEquals("Test 44", "16.01.2009 08:58:59,999 false", strings[5]);
    assertEquals("Test 45", "16.01.2009 15:31:00,000 true", strings[6]);
    assertEquals("Test 46", "16.01.2009 17:43:59,999 false", strings[7]);

  }

  @Test
  public void setGueltigkeitVerkehrTest4() throws Exception
  {
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit2");
    
    Date d1 = sdf.parse("15.01.2009 00:00:00,000");
    Date d2 = sdf.parse("16.01.2009 23:59:59,999");
    
    SortedMap<Long, Boolean> tmp = ske.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime());

    SortedMap<Long, Boolean> sm = _listeZustandsWechsel.setGueltigkeitVerkehr(tmp, 60 * 1000,
        1, 60 * 1000, 3);

    String[] strings = new String[sm.entrySet().size()];

    int cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();

      cnt++;
    }

    assertTrue("Test 47", strings.length == 8);
    assertEquals("Test 48", "15.01.2009 07:16:00,000 true", strings[0]);
    assertEquals("Test 49", "15.01.2009 09:00:59,999 false", strings[1]);
    assertEquals("Test 50", "15.01.2009 15:31:00,000 true", strings[2]);
    assertEquals("Test 51", "15.01.2009 17:45:59,999 false", strings[3]);
    assertEquals("Test 52", "16.01.2009 07:16:00,000 true", strings[4]);
    assertEquals("Test 53", "16.01.2009 09:00:59,999 false", strings[5]);
    assertEquals("Test 54", "16.01.2009 15:31:00,000 true", strings[6]);
    assertEquals("Test 55", "16.01.2009 17:45:59,999 false", strings[7]);

  }

  @Test
  public void setGueltigkeitVerkehrTest5() throws Exception
  {
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    Map<String, SystemkalenderEintrag> skeList = EreignisKalender.getSkeList();

    SystemkalenderEintrag ske = skeList.get("ske.hauptverkehrszeit2");
    
    Date d1 = sdf.parse("15.01.2009 00:00:00,000");
    Date d2 = sdf.parse("16.01.2009 23:59:59,999");
    
    SortedMap<Long, Boolean> tmp = ske.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime());
    

    SortedMap<Long, Boolean> sm = _listeZustandsWechsel
        .setGueltigkeitVerkehr(tmp, 0, 0, 0, 0);

    String[] strings = new String[sm.entrySet().size()];

    int cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();

      cnt++;
    }

    assertTrue("Test 56", strings.length == 8);
    assertEquals("Test 57", "15.01.2009 07:15:00,000 true", strings[0]);
    assertEquals("Test 58", "15.01.2009 08:59:59,999 false", strings[1]);
    assertEquals("Test 59", "15.01.2009 15:30:00,000 true", strings[2]);
    assertEquals("Test 60", "15.01.2009 17:44:59,999 false", strings[3]);
    assertEquals("Test 61", "16.01.2009 07:15:00,000 true", strings[4]);
    assertEquals("Test 62", "16.01.2009 08:59:59,999 false", strings[5]);
    assertEquals("Test 63", "16.01.2009 15:30:00,000 true", strings[6]);
    assertEquals("Test 64", "16.01.2009 17:44:59,999 false", strings[7]);

    _listeZustandsWechsel = new ListeZustandsWechsel();

    sm = _listeZustandsWechsel.setGueltigkeitVerkehr(tmp, 0, 3, 0, 3);

    strings = new String[sm.entrySet().size()];

    cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();

      cnt++;
    }
    
    assertTrue("Test 65", strings.length == 8);
    assertEquals("Test 66", "15.01.2009 07:15:00,000 true", strings[0]);
    assertEquals("Test 67", "15.01.2009 08:59:59,999 false", strings[1]);
    assertEquals("Test 68", "15.01.2009 15:30:00,000 true", strings[2]);
    assertEquals("Test 69", "15.01.2009 17:44:59,999 false", strings[3]);
    assertEquals("Test 70", "16.01.2009 07:15:00,000 true", strings[4]);
    assertEquals("Test 71", "16.01.2009 08:59:59,999 false", strings[5]);
    assertEquals("Test 72", "16.01.2009 15:30:00,000 true", strings[6]);
    assertEquals("Test 73", "16.01.2009 17:44:59,999 false", strings[7]);

    _listeZustandsWechsel = new ListeZustandsWechsel();

    sm = _listeZustandsWechsel.setGueltigkeitVerkehr(tmp, 0, 3, 60 * 1000, 0);

    strings = new String[sm.entrySet().size()];

    cnt = 0;
    for (Map.Entry<Long, Boolean> me : sm.entrySet())
    {
      Date d = new Date();
      d.setTime(me.getKey());
      strings[cnt] = sdf.format(d) + " " + me.getValue();

      cnt++;
    }

    assertTrue("Test 74", strings.length == 8);
    assertEquals("Test 75", "15.01.2009 07:14:00,000 false", strings[0]);
    assertEquals("Test 76", "15.01.2009 07:15:00,000 true", strings[1]);
    assertEquals("Test 77", "15.01.2009 15:29:00,000 false", strings[2]);
    assertEquals("Test 78", "15.01.2009 15:30:00,000 true", strings[3]);
    assertEquals("Test 79", "16.01.2009 07:14:00,000 false", strings[4]);
    assertEquals("Test 80", "16.01.2009 07:15:00,000 true", strings[5]);
    assertEquals("Test 81", "16.01.2009 15:29:00,000 false", strings[6]);
    assertEquals("Test 82", "16.01.2009 15:30:00,000 true", strings[7]);

  }

}
